#!/usr/bin/env python3
'''
Crea diapositivas PDF desde un SVG siguiendo  estas reglas:
Las capas cuyo nombre empieza por ALL: aparecen en todas las diapos
Las capas cuyo nombre empieza por NOT: no aparecen en ninguna diapo
Por cada capa que empiece por D: se creará una diapositiva
'''
import argparse
import os
import subprocess
import sys
from xml.etree import cElementTree as ElementTree


def main():
    parser = argparse.ArgumentParser(description='diapo_pdf')
    parser.add_argument('input_file', help='file to split')
    parser.add_argument('--dpi', type=int, default=300, help='DPI for export (default: 300)')
    #parser.add_argument('--layers', nargs='*', help='layers to export (default: all layers)')

    args = parser.parse_args()

    filename = args.input_file
    dpi = args.dpi
    #layers = args.layers
    
    doc = ElementTree.parse(filename)
    root = doc.getroot()
    
    inkscape = '{http://www.inkscape.org/namespaces/inkscape}'
    svg      = '{http://www.w3.org/2000/svg}'
    
    
    dlist=[]
    print('Capas encontradas:')
    print('| {:20s} | {:20s} |'.format('id','label'))
    for elem in root.iterfind('{:}g[@{:}groupmode="layer"]'.format(svg,inkscape)):
        l_label = elem.attrib[inkscape+'label']
        l_id    = elem.attrib['id']
        print('| {:20s} | {:20s} |'.format(l_id,l_label))
        #print(elem.attrib['style'])
        if elem.attrib[inkscape+'label'][0:2]=='D:':
            dlist.append(l_label)
    print(' ')
    
    for elem in root.iterfind('{:}g[@{:}groupmode="layer"]'.format(svg,inkscape)):
        l_label=elem.attrib[inkscape+'label']
        if l_label.split(':')[0]=='ALL':
            elem.attrib['style'] =  disp_on(elem.attrib['style'])
        if l_label.split(':')[0]=='NOT':
            elem.attrib['style'] = disp_off(elem.attrib['style'])
        if l_label.split(':')[0]=='D':
            elem.attrib['style'] = disp_off(elem.attrib['style'])
    
    tmpdir='tmpdir_{:}'.format(os.getpid())
    os.mkdir(tmpdir)
    
    cmd='inkscape --without-gui --export-pdf=./'+tmpdir+'/d{:04d}.pdf --export-dpi=300  --export-area-page {:}'
    
    ff=[]
    for i,D in enumerate(dlist):
        print("Procesando {:}".format(D))
        for elem in root.iterfind('{:}g[@{:}groupmode="layer"]'.format(svg,inkscape)):
            if elem.attrib[inkscape+'label']==D:
                elem.attrib['style']=disp_on(elem.attrib['style'])
                doc.write(tmpdir+'/tmp.svg')
                subprocess.call(cmd.format(i,tmpdir+'/tmp.svg').split() )
                ff.append('./'+tmpdir+'/d{:04d}.pdf'.format(i))
                elem.attrib['style']=disp_off(elem.attrib['style'])
                os.remove(tmpdir+'/tmp.svg')
    print(' ')
    print("Creando el PDF")
    gscmd='gs -dBATCH -dNOPAUSE -q -sDEVICE=pdfwrite -sOutputFile={:} '+' '.join(ff)
    subprocess.call( gscmd.format( filename[0:-4]+'_.pdf' ).split() )
    
    print(' ')
    print("Limpiando temporales")
    
    for i in ff:
        os.remove(i)
    os.rmdir(tmpdir)
    
    return 0


def disp_on(ss):
    out=[]
    if 'display' in ss:
        for i in ss.split(';'):
            if i.split(':')[0]=='display':
                out.append('display:inline')
            else:
                out.append(i)
        return ';'.join(out)
    else:
        return ss+';display:inline'

def disp_off(ss):
    out=[]
    if 'display' in ss:
        for i in ss.split(';'):
            if i.split(':')[0]=='display':
                out.append('display:none')
            else:
                out.append(i)
        return ';'.join(out)
    else:
        return ss+';display:none'


############



if __name__ == '__main__':
    main()
