# -*- coding: utf-8 -*-
"""
Created on Thu Mar 21 14:05:30 2019

@author: lolo
"""

from numpy import *
import matplotlib.pyplot as plt


#%%

fig = plt.figure(figsize=(8,5))
ax = fig.add_subplot(111)


random.seed(6)

xx     = linspace(-5,5,10)
y1     = xx*0.8 + 2.3 + random.normal(size=10)*0.5
y1_err = (1+random.normal(size=10)*0.3)

ax.plot(xx,y1,'o' , color='C0', alpha=1, label='datos')
#(_, caps, _) = ax.errorbar(xx,y1, yerr=y1_err, fmt='.', markersize=0, capsize=3, color='C0')
(_, caps, _) = ax.errorbar(xx,y1, yerr=y1_err, fmt='o', markersize=0, capsize=3, color='C0')

for cap in caps:
    cap.set_markeredgewidth(1)

ax.set_xlabel('x')
ax.set_ylabel('y')

ax.grid(b=True,linestyle='--',color='lightgray')
ax.set_yticks( arange(-9,13,3) )
ax.set_ylim(-3,9)
ax.set_xlim(-5.2,5.2)

ax.set_title(r'Modelo:  $f_{a,b}(x) = x \cdot a + b $')

fig.savefig('g00.png')


#%%

x2 = linspace(-5.5,5.5,10)

ax.plot(x2, x2*1+2.5      ,'-' , label='a=2,b=2.5' , color='C1' , alpha=0.7 )
ax.plot(x2, x2*0.2+2      ,'-' , label='a=1,b=2' , color='C2' , alpha=0.7 )
ax.plot(x2, x2*5+0      ,'-' , label='a=5,b=0' , color='C3' , alpha=0.7 )
ax.plot(x2, -x2*0.5+6   ,'-' , label='a=-1/2,b=6', color='C4' , alpha=0.7 )

ax.legend(loc='best')

fig.savefig('s01.svg')

fig.savefig('g01.png')

#%%

fig = plt.figure(figsize=(8,5))
ax = fig.add_subplot(111)

y2 = xx*1+2.5

for X,Y1,Y2 in zip(xx,y1,y2):
    ax.plot([X]*2,[Y1,Y2],'-',color='red',alpha=1)

ax.plot(xx,y1,'o' , color='C0', alpha=1, label='datos')
ax.plot(x2, x2*1+2.5      ,'-' , label='y=x*2+3' , color='C1' , alpha=0.7 )
ax.plot(xx,     y2      ,'.' , label='y=x*2+3' , color='C1' )



#(_, caps, _) = ax.errorbar(xx,y1, yerr=y1_err, fmt='.', markersize=0, capsize=3, color='C0')
#(_, caps, _) = ax.errorbar(xx,y1, yerr=y1_err, fmt='o', markersize=0, capsize=3, color='C0')

#for cap in caps:
#    cap.set_markeredgewidth(1)

ax.set_xlabel('x')
ax.set_ylabel('y')

ax.grid(b=True,linestyle='--',color='lightgray')
ax.set_yticks( arange(-9,13,3) )
ax.set_ylim(-3,9)
ax.set_xlim(-5.2,5.2)

ax.set_title(r'Residuos')


fig.savefig('s02.png')

fig.savefig('s02.svg')


#%%



from mpl_toolkits.mplot3d import axes3d
import matplotlib.pyplot as plt

fig = plt.figure(figsize=(10,7))
ax = fig.add_subplot(111, projection='3d')

# load some test data for demonstration and plot a wireframe
#X, Y, Z = axes3d.get_test_data(0.1)

N=25

def modelo(x,a,b):
    return  [ w*a+b  for w in x ] 


def chi2(a,b):
    Z=0
    for ww,mod in zip(y1,modelo(xx,a,b)):
        Z+= ( ww - mod )**2
    return Z


r = linspace(0, 1, N)
p = linspace(0, 2*pi, N)
R, P = meshgrid(r, p)

# Express the mesh in the cartesian system.
A, B = 0.8+R*cos(P), 2.3+R*sin(P)*4


#A = outer(linspace(0, 2, N), ones(N))
#B = outer(linspace(0, 4, N), ones(N)).copy().T
Z = chi2(A,B)

ax.plot_wireframe(A, B, Z, rstride=2, cstride=2, color='black', alpha=0.5)
ax.plot_surface(A, B, Z, cmap=plt.cm.YlGnBu_r , alpha=0.5)
ax.contourf(    A, B, Z, zdir='z', offset=-50, cmap=plt.cm.YlGnBu_r , alpha=0.5)
ax.plot(        [0.8]*2, [B.min(),B.max()], zs=-50, zdir='z', color='black')
ax.plot(    [A.min(),A.max()],[2.3]*2, zs=-50, zdir='z', color='black')

ax.view_init(24, -60)

ax.set_xticks( arange(0,2.1,0.5))
ax.set_yticks( arange(-1,6.1,2))

ax.set_zlim(-50,180)
ax.set_xlabel('A')
ax.set_ylabel('B')
ax.set_zlabel(r'$\chi^2$')

fig.savefig('s03.png')
fig.savefig('s03.svg')

#%%




ax.plot(   [1],[2.5]  ,'o' , zs=-50, zdir='z', label='a=2,b=2.5' , color='C1' , alpha=1 )
ax.plot( [0.2],[2]    ,'o' , zs=-50, zdir='z', label='a=1,b=2' , color='C2' , alpha=1 )
#ax.plot(   [5],[0]    ,'o' , zs=-50, zdir='z', label='a=5,b=0' , color='C3' , alpha=1 )
ax.plot([-0.5],[6]    ,'o' , zs=-50, zdir='z', label='a=-1/2,b=6', color='C4' , alpha=1 )

#ax.plot(         [-1,2],[1]*2, zs=-50, zdir='z', color='black')

ax.view_init(24, -60)

ax.set_xticks( arange(0,2.1,0.5))
ax.set_yticks( arange(-1,6.1,2))

ax.set_zlim(-50,180)
ax.set_xlabel('A')
ax.set_ylabel('B')
ax.set_zlabel(r'$\chi^2$')



fig.savefig('s04.png')
fig.savefig('s04.svg')



#%%


fig = plt.figure(figsize=(8,5))
ax = fig.add_subplot(111)


random.seed(6)

xx     = linspace(-5,5,10)
y1     = xx*0.8 + 2.3 + random.normal(size=10)*0.5
y1_err = (1+random.normal(size=10)*0.3)

mm1 , cv1  = polyfit(xx,y1,1,cov=True )
ym1        = polyval(mm1,x2)

mm2 , cv2  = polyfit(xx,y1,1,cov=True , w=1/y1_err)
ym2        = polyval(mm1,x2)

ax.plot(xx,y1,'o' , color='C0', alpha=1, label='datos')
#(_, caps, _) = ax.errorbar(xx,y1, yerr=y1_err, fmt='.', markersize=0, capsize=3, color='C0')
(_, caps, _) = ax.errorbar(xx,y1, yerr=y1_err, fmt='o', markersize=0, capsize=3, color='C0')

ax.plot(x2,ym1,'-' , color='C1', alpha=1, label='modelo')
#ax.plot(x2,ym2,':' , color='red', alpha=0.5, label='modelo\ncon errores')


for cap in caps:
    cap.set_markeredgewidth(1)

ax.set_xlabel('x')
ax.set_ylabel('y')

ax.grid(b=True,linestyle='--',color='lightgray')
ax.set_yticks( arange(-9,13,3) )
ax.set_ylim(-3,9)
ax.set_xlim(-5.2,5.2)

ax.set_title(r'Modelo:  $f_{a,b}(x) = x \cdot a + b $')

fig.savefig('g05.png')
fig.savefig('g05.svg')


for par,val,err in zip('a,b'.split(','),mm1,sqrt(diag(cv1))):
    print( par ,' = ', val , '±' , err)


for par,val,err in zip('a,b'.split(','),mm2,sqrt(diag(cv2))):
    print( par ,' = ', val , '±' , err)





#%%


for i in range(10):
    random.seed(i)
    
    xx     = linspace(-5,5,10)
    y1     = xx*0.8 + 2.3 + random.normal(size=10)*0.5
    y1_err = (1+random.normal(size=10)*0.3)
    
    mm2 , cv2  = polyfit(xx,y1,1,cov=True , w=1/y1_err)
    ym2        = polyval(mm1,x2)
    
    print( i , ' | ', mm2 - array([0.8,2.3]) )



#%%

for x,y,err in zip(xx,y1,y1_err):
    print( '{: 5.4f}\t{: 5.4f}\t{: 5.4f}'.format(x,y,err) )








#%%


#%%


fig = plt.figure(figsize=(8,5))
ax = fig.add_subplot(111)


random.seed(6)

L     = linspace(10,70,6)/100
g     = 9.8
T     = 2*pi*sqrt(L/g) + (random.normal(size=6)*0.05)
T_err = (1+random.normal(size=6)*0.3)*0.1


Lm     = linspace(0.1,80,1000)/100
Tm     = 2*pi*sqrt(Lm/g)



ax.plot(Lm*100,Tm,'-' , color='red', alpha=1, label='modelo')

#(_, caps, _) = ax.errorbar(xx,y1, yerr=y1_err, fmt='.', markersize=0, capsize=3, color='C0')
ax.plot(L*100,T,'o' , color='C0', alpha=1, label='modelo')
(_, caps, _) = ax.errorbar(L*100,T, yerr=T_err, fmt='o', markersize=0, capsize=3, color='C0')

#ax.plot(x2,ym2,':' , color='red', alpha=0.5, label='modelo\ncon errores')


for cap in caps:
    cap.set_markeredgewidth(1)

ax.set_xlabel('L [cm]')
ax.set_ylabel('T [s]')

ax.set_yticks( arange(0,2.1,0.5))

ax.grid(b=True,linestyle='--',color='lightgray')
#ax.set_yticks( arange(-9,13,3) )
ax.set_ylim(0,2)
ax.set_xlim(0,80)

ax.set_title(r'Modelo:  $T(L) = 2\,\pi \cdot \sqrt{\frac{L}{g}} $')

#fig.savefig('g05.png')
#fig.savefig('g05.svg')








#%%




from mpl_toolkits.mplot3d import axes3d
import matplotlib.pyplot as plt

fig = plt.figure()
ax = fig.add_subplot(111, projection='3d')

# load some test data for demonstration and plot a wireframe
#X, Y, Z = axes3d.get_test_data(0.1)



def fun(X,Y):
    return (sin( (Y/15) ** 2) + 1.5*cos((X/10) ** 2))*(Y/30+3)

X = outer(linspace(-30, 30, 100), ones(100))
Y = X.copy().T
Z = fun(X,Y)

x0=10
y0=-10


x1 = linspace(-30, 30, 100)
y1 = ones(100)*y0
z1 = fun(x1,y1)
dz1= polyval(polyfit( linspace(x0-0.01,x0+0.01), fun(linspace(x0-0.01,x0+0.01),ones(50)*y0),1), x1)

y2 = linspace(-30, 30, 100)
x2 = ones(100)*x0
z2 = fun(x2,y2)
dz2= polyval(polyfit( linspace(y0-0.01,y0+0.01), fun(ones(50)*x0,linspace(y0-0.01,y0+0.01)),1), y2)


ax.plot_wireframe(X, Y, Z, rstride=2, cstride=2, alpha=0.5)

ax.plot(x1,y1,z1  , color='black', linewidth=3)
ax.plot(x1,y1,dz1 , color='red', linewidth=2 , alpha=0.5)

#ax.plot(x2,y2,z2 , color='black', linewidth=3)
#ax.plot(x2,y2,dz2 , color='red', linewidth=2 , alpha=0.5)

#plt.hold(True)
#ax.scatter(x0,y0,fun(x0,y0), s=100 , c='red')


ax.view_init(32, -50)

ax.set_zlim(-7.5,7.5)
ax.set_xlabel('x')
ax.set_ylabel('y')


#%%


from mpl_toolkits.mplot3d import axes3d
import matplotlib.pyplot as plt

fig = plt.figure()
ax = fig.add_subplot(111, projection='3d')

# load some test data for demonstration and plot a wireframe
#X, Y, Z = axes3d.get_test_data(0.1)



def fun(X,Y):
    return (sin( (Y/15) ** 2) + 1.5*cos((X/10) ** 2))*(Y/30+3)

X = outer(linspace(-30, 30, 100), ones(100))
Y = X.copy().T
Z = fun(X,Y)

x0=10
y0=-10


x1 = linspace(-30, 30, 100)
y1 = ones(100)*y0
z1 = fun(x1,y1)
dz1= polyval(polyfit( linspace(x0-0.01,x0+0.01), fun(linspace(x0-0.01,x0+0.01),ones(50)*y0),1), x1)

y2 = linspace(-30, 30, 100)
x2 = ones(100)*x0
z2 = fun(x2,y2)
dz2= polyval(polyfit( linspace(y0-0.01,y0+0.01), fun(ones(50)*x0,linspace(y0-0.01,y0+0.01)),1), y2)


ax.plot_wireframe(X, Y, Z, rstride=2, cstride=2, alpha=0.5)

#ax.plot(x1,y1,z1  , color='black', linewidth=3)
#ax.plot(x1,y1,dz1 , color='red', linewidth=2 , alpha=0.5)

ax.plot(x2,y2,z2 , color='black', linewidth=3)
ax.plot(x2,y2,dz2 , color='red', linewidth=2 , alpha=0.5)

#plt.hold(True)
#ax.scatter(x0,y0,fun(x0,y0), s=100 , c='red')


ax.view_init(32, -50)

ax.set_zlim(-7.5,7.5)
ax.set_xlabel('x')
ax.set_ylabel('y')



#%%

fig = plt.figure(figsize=(8,5))

#%%
#fig = plt.figure(figsize=(8,5))
fig.clf()
ax = fig.add_subplot(111)

def fun(X):
    return tan((array(X)-5)/3.5)+8

xx = linspace(0,3,200)
yy = fun(xx)
y0 = zeros(200)

ax.plot(xx,yy , color='C0', linewidth=3)
ax.set_xlabel('x')
ax.set_ylabel('y')

ax.set_title('Propagación ABSOLUTA sin aproximar')

ax.grid(b=True,linestyle='--',color='lightgray')
ax.set_ylim(0,8)
ax.set_xlim(0,3)


ax.plot([1,1,0], [0,fun(1),fun(1)], color='C0')
#ax.plot([1], [fun(1)], 'o' ,  color='C0')

fig.savefig('prop_00.png')

ax.fill_between(xx,y0,yy,where=(xx>0.5)&(xx<1.5) , color='C0', alpha=0.5)
ax.fill_betweenx(yy,y0,xx,where=(xx>0.5)&(xx<1.5), color='C0', alpha=0.5)

fig.savefig('prop_00b.png')

#%%
fig.clf()
ax = fig.add_subplot(111)

def fun(X):
    return tan((array(X)-5)/3.5)+8

xx = linspace(0,3,200)
yy = fun(xx)
y0 = zeros(200)

ax.plot(xx,yy , color='C0', linewidth=3)
ax.set_xlabel('x')
ax.set_ylabel('y')

ax.set_title(r'Propagación ABSOLUTA aproximada a $1^{er}$ orden')

ax.grid(b=True,linestyle='--',color='lightgray')
ax.set_ylim(0,8)
ax.set_xlim(0,3)

mm = polyfit( linspace(0.999,1.001), fun(linspace(0.999,1.001)) ,1)
yl = polyval(mm,xx)

def dfun(X):
    return polyval(mm,array(X))

ax.plot(xx,yl,color='red')
ax.plot([1,1,0], [0,fun(1),fun(1)], color='C1')

fig.savefig('prop_01.png')

ax.fill_between(xx,y0,yl,where=(xx>0.5)&(xx<1.5) , color='C1', alpha=0.5)
ax.fill_betweenx(yl,y0,xx,where=(xx>0.5)&(xx<1.5), color='C1', alpha=0.5)

fig.savefig('prop_02.png')


ax.plot([0.5,0.5,0], [0,fun(0.5),fun(0.5)] , color='C0')
ax.plot([1.5,1.5,0], [0,fun(1.5),fun(1.5)] , color='C0')

fig.savefig('prop_03.png')

#%%
fig.clf()
ax = fig.add_subplot(111)

def fun(X):
    return tan((array(X)-5)/3.5)+8

xx = linspace(0,3,200)
yy = fun(xx)
y0 = zeros(200)

ax.plot(xx,yy , color='C0', linewidth=3)
ax.set_xlabel('x')
ax.set_ylabel('y')

ax.set_title('Propagación ESTADÍSTICA (Error Estandar)')

ax.grid(b=True,linestyle='--',color='lightgray')
ax.set_ylim(0,8)
ax.set_xlim(0,3)

mm = polyfit( linspace(0.999,1.001), fun(linspace(0.999,1.001)) ,1)
yl = polyval(mm,xx)

def dfun(X):
    if type(X) == float:
        X=[X]
    return polyval(mm,array(X))

def gauss(X,x0,s):
    return 1./sqrt(2*pi)/s*exp(- (array(X)-x0)**2/(2*s**2))   * sqrt(2*pi)*s

ax.plot(xx,yl,color='red')
ax.plot([1,1,0], [0,fun(1),fun(1)], color='C1')

ax.plot(xx,gauss(xx,1,0.5) , color='C1', alpha=0.5)
yx = linspace(0,8,200)
ax.plot(gauss(yx,fun(1),0.5*mm[0])/4,yx , color='C1', alpha=0.5)

fig.savefig('prop_04.png')


Z1 = array([ gauss(xx,1,0.5) for y in yx  ])

Z1 = zeros((200,200))

for iy,y in enumerate(yx):
    for ix,x in enumerate(xx):
        Z1[iy,ix] = gauss(x,1,0.5) if polyval(mm,x)>y else gauss(y,fun(1),0.5*mm[0])

im = ax.imshow(Z1, interpolation='bilinear', cmap=plt.cm.Reds,
               origin='lower', extent=[0,3, 0, 8], aspect="auto",
               vmax=abs(Z1).max()*4, vmin=abs(Z1).min())

fig.savefig('prop_05.png')

ax.plot([0.5,0.5,0], [0,dfun(0.5),dfun(0.5)] , color='C1')
ax.plot([1.5,1.5,0], [0,dfun(1.5),dfun(1.5)] , color='C1')


fig.savefig('prop_06.png')

#ax.fill_between(xx,y0,yl,where=(xx>0.5)&(xx<1.5) , color='C1', alpha=0.5)
#ax.fill_betweenx(yl,y0,xx,where=(xx>0.5)&(xx<1.5), color='C1', alpha=0.5)

#ax.plot([0.5,0.5,0], [0,fun(0.5),fun(0.5)] , color='C0')
#ax.plot([1.5,1.5,0], [0,fun(1.5),fun(1.5)] , color='C0')


#%%

#fig = plt.figure(figsize=(8,5))
fig.clf()
ax = fig.add_subplot(111)


def gauss(X,x0,s):
    return 1./sqrt(2*pi)/s*exp(- (array(X)-x0)**2/(2*s**2))


xx = linspace(0,4,300)
y1 = gauss(xx,2.3,0.3)
y2 = gauss(xx,1.8,0.6)
y3 = gauss(xx,3,0.1)

(_, caps, _) = ax.errorbar(2.3, 4, xerr=0.3, fmt='o', markersize=8, capsize=10, color='C0')
for cap in caps:
    cap.set_markeredgewidth(2)

(_, caps, _) = ax.errorbar(1.8, 4.5, xerr=0.6, fmt='o', markersize=8, capsize=10, color='C1')
for cap in caps:
    cap.set_markeredgewidth(2)

(_, caps, _) = ax.errorbar(3, 5, xerr=0.1, fmt='o', markersize=8, capsize=10, color='C2')
for cap in caps:
    cap.set_markeredgewidth(2)

y0 = zeros(len(xx))

ax.plot(xx, y1 , color='C0', linewidth=2, label='$2.3 \pm 0.3$')
ax.fill_between(xx,y0,y1,where=(xx>2.3-0.3)&(xx<2.3+0.3) , color='C0', alpha=0.5)

ax.plot(xx,y2 , color='C1', linewidth=2, label='$1.8 \pm 0.6$')
ax.fill_between(xx,y0,y2,where=(xx>1.8-0.6)&(xx<1.8+0.6) , color='C1', alpha=0.5)

ax.plot(xx,y3 , color='C2', linewidth=2, label='$3.0 \pm 0.1$')
ax.fill_between(xx,y0,y3,where=(xx>3.0-0.1)&(xx<3.0+0.1) , color='C2', alpha=0.5)

ax.set_yticks([])

ax.legend(loc=6)

ax.set_xlabel('x')
ax.set_ylabel('')

ax.set_title('Discrepancias')

ax.grid(b=True,linestyle='--',color='lightgray')
ax.set_ylim(0,5.5)
ax.set_xlim(0,4)



fig.savefig('discrepancia.svg')


#%%
random.seed(1)

import numpy as np
import matplotlib.pyplot as plt
from matplotlib.ticker import NullFormatter


def gauss(X,x0,s):
    return 1./sqrt(2*pi)/s*exp(- (array(X)-x0)**2/(2*s**2))


# the random data
x = np.random.randn(200)
y = np.arange(200)

nullfmt = NullFormatter()         # no labels

# definitions for the axes
left, width = 0.1, 0.65
bottom, height = 0.1, 0.5
bottom_h = left_h = left + 0.5 + 0.02

rect_scatter = [left, bottom, width, height]
rect_histx = [left, bottom_h, width, 0.3]
rect_histy = [left_h, bottom, 0.2, height]

# start with a rectangular Figure
plt.figure(1, figsize=(8, 5))

axScatter = plt.axes(rect_scatter)
axHistx = plt.axes(rect_histx)
#axHisty = plt.axes(rect_histy)

# no labels
axHistx.xaxis.set_major_formatter(nullfmt)
#axHisty.yaxis.set_major_formatter(nullfmt)

# the scatter plot:
axScatter.scatter(x, y, marker='.')

# now determine nice limits by hand:
binwidth = 0.25
#xymax = np.max([np.max(np.fabs(x)), np.max(np.fabs(y))])
xmax = np.max(np.fabs(x))
ymax = np.max(np.fabs(y))

limx = (int(xmax/binwidth) + 1) * binwidth
limy = (int(ymax/binwidth) + 1) * binwidth



axScatter.set_xlim((-limx, limx))
axScatter.set_ylim((0, limy))
axScatter.set_ylabel('Num de medicion')



bins = np.arange(-limx, limx + binwidth, binwidth)

xx = linspace(-limx,limx)
axHistx.hist(x, bins=bins)
axHistx.plot( xx , gauss(xx, mean(x), std(x) )*sum(x)/mean(diff(xx))*binwidth )
#axHisty.hist(y, bins=bins, orientation='horizontal')

axHistx.set_xlim(axScatter.get_xlim())
#axHisty.set_ylim(axScatter.get_ylim())

axScatter.set_xticks(bins)
axHistx.set_xticks(bins)

axScatter.grid(b=True,axis='x')
axHistx.grid(b=True,axis='x')

axScatter.set_xticklabels( [ str(y/4.0) if y%4==0 else ''  for y in (bins*4).astype(int) ] )
axScatter.set_xlabel('Valor medido')

plt.show()
plt.savefig('hist.png')

#plt.tight_layout()

#%%
random.seed(1)

xx = random.normal(size=1000)

plt.figure()

plt.subplot(311)
plt.title('1000 mediciones')

plt.hist(xx)
plt.ylabel('# bins = 10')

plt.subplot(312)
plt.hist(xx,30)
plt.ylabel('# bins = 30')

plt.subplot(313)
plt.hist(xx,100)
plt.ylabel('# bins = 100')



#%%
random.seed(1)
xx = random.normal(size=1000)

plt.figure()
plt.hist(xx,20)
plt.xlabel('Valor')
plt.ylabel('# de apariciones')











